'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

// Copyright Joyent, Inc. and other Node contributors.
//
// Permission is hereby granted, free of charge, to any person obtaining a
// copy of this software and associated documentation files (the
// "Software"), to deal in the Software without restriction, including
// without limitation the rights to use, copy, modify, merge, publish,
// distribute, sublicense, and/or sell copies of the Software, and to permit
// persons to whom the Software is furnished to do so, subject to the
// following conditions:
//
// The above copyright notice and this permission notice shall be included
// in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
// OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN
// NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
// DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
// OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE
// USE OR OTHER DEALINGS IN THE SOFTWARE.


// If obj.hasOwnProperty has been overridden, then calling
// obj.hasOwnProperty(prop) will break.
// See: https://github.com/joyent/node/issues/1707
function hasOwnProperty(obj, prop) {
  return Object.prototype.hasOwnProperty.call(obj, prop);
}
var isArray = Array.isArray || function (xs) {
  return Object.prototype.toString.call(xs) === '[object Array]';
};
function stringifyPrimitive(v) {
  switch (typeof v) {
    case 'string':
      return v;

    case 'boolean':
      return v ? 'true' : 'false';

    case 'number':
      return isFinite(v) ? v : '';

    default:
      return '';
  }
}

function stringify (obj, sep, eq, name) {
  sep = sep || '&';
  eq = eq || '=';
  if (obj === null) {
    obj = undefined;
  }

  if (typeof obj === 'object') {
    return map(objectKeys(obj), function(k) {
      var ks = encodeURIComponent(stringifyPrimitive(k)) + eq;
      if (isArray(obj[k])) {
        return map(obj[k], function(v) {
          return ks + encodeURIComponent(stringifyPrimitive(v));
        }).join(sep);
      } else {
        return ks + encodeURIComponent(stringifyPrimitive(obj[k]));
      }
    }).join(sep);

  }

  if (!name) return '';
  return encodeURIComponent(stringifyPrimitive(name)) + eq +
         encodeURIComponent(stringifyPrimitive(obj));
}
function map (xs, f) {
  if (xs.map) return xs.map(f);
  var res = [];
  for (var i = 0; i < xs.length; i++) {
    res.push(f(xs[i], i));
  }
  return res;
}

var objectKeys = Object.keys || function (obj) {
  var res = [];
  for (var key in obj) {
    if (Object.prototype.hasOwnProperty.call(obj, key)) res.push(key);
  }
  return res;
};

function parse(qs, sep, eq, options) {
  sep = sep || '&';
  eq = eq || '=';
  var obj = {};

  if (typeof qs !== 'string' || qs.length === 0) {
    return obj;
  }

  var regexp = /\+/g;
  qs = qs.split(sep);

  var maxKeys = 1000;
  if (options && typeof options.maxKeys === 'number') {
    maxKeys = options.maxKeys;
  }

  var len = qs.length;
  // maxKeys <= 0 means that we should not limit keys count
  if (maxKeys > 0 && len > maxKeys) {
    len = maxKeys;
  }

  for (var i = 0; i < len; ++i) {
    var x = qs[i].replace(regexp, '%20'),
        idx = x.indexOf(eq),
        kstr, vstr, k, v;

    if (idx >= 0) {
      kstr = x.substr(0, idx);
      vstr = x.substr(idx + 1);
    } else {
      kstr = x;
      vstr = '';
    }

    k = decodeURIComponent(kstr);
    v = decodeURIComponent(vstr);

    if (!hasOwnProperty(obj, k)) {
      obj[k] = v;
    } else if (isArray(obj[k])) {
      obj[k].push(v);
    } else {
      obj[k] = [obj[k], v];
    }
  }

  return obj;
}var qs = {
  encode: stringify,
  stringify: stringify,
  decode: parse,
  parse: parse
};

var classCallCheck = function (instance, Constructor) {
  if (!(instance instanceof Constructor)) {
    throw new TypeError("Cannot call a class as a function");
  }
};

var createClass = function () {
  function defineProperties(target, props) {
    for (var i = 0; i < props.length; i++) {
      var descriptor = props[i];
      descriptor.enumerable = descriptor.enumerable || false;
      descriptor.configurable = true;
      if ("value" in descriptor) descriptor.writable = true;
      Object.defineProperty(target, descriptor.key, descriptor);
    }
  }

  return function (Constructor, protoProps, staticProps) {
    if (protoProps) defineProperties(Constructor.prototype, protoProps);
    if (staticProps) defineProperties(Constructor, staticProps);
    return Constructor;
  };
}();

/**
 * Convert all fields in a payload dictionary to strings
 *
 * @param {object} payload Payload on which the new dictionary is based
 */
function valuesToStrings(payload) {
  var stringifiedPayload = {};
  for (var key in payload) {
    if (payload.hasOwnProperty(key)) {
      stringifiedPayload[key] = payload[key].toString();
    }
  }
  return stringifiedPayload;
}

var Emitter = function () {
  /**
   * Create an emitter object which will send events to a collector
   *
   * @param endpoint
   * @param protocol
   * @param port
   * @param method
   * @param bufferSize
   * @param callback
   */
  function Emitter(endpoint, protocol, port, method, bufferSize, callback) {
    classCallCheck(this, Emitter);

    this.protocol = (protocol || 'http').toLowerCase();
    this.method = (method || 'get').toLowerCase();
    this.bufferSize = bufferSize;
    if (bufferSize === null || typeof bufferSize === 'undefined') {
      this.bufferSize = method === 'get' ? 0 : 10;
    }
    this.callback = callback;

    var portString = port ? ':' + port : '';
    var path = method === 'get' ? '/i' : '/com.snowplowanalytics.snowplow/tp2';
    this.targetUrl = protocol + '://' + endpoint + portString + path;

    this.buffer = [];
  }

  /**
   * Send all events queued in the buffer to the collector
   */


  createClass(Emitter, [{
    key: 'flush',
    value: function flush() {
      var _this = this;

      var temp = this.buffer;
      this.buffer = [];

      if (this.method === 'post') {
        var postJson = {
          schema: 'iglu:com.snowplowanalytics.snowplow/payload_data/jsonschema/1-0-0',
          data: temp.map(valuesToStrings)
        };

        fetch(this.targetUrl, {
          method: 'POST',
          headers: {
            'content-type': 'application/json; charset=utf-8'
          },
          body: JSON.stringify(postJson)
        }).then(function (response) {
          return _this.handleResponseAndCallback(response);
        }).catch(function (e) {
          _this.callback(e);
        });
      } else {
        for (var i = 0; i < temp.length; i++) {
          var url = this.targetUrl + '?' + qs.stringify(temp[i]);

          fetch(url, {
            method: 'GET'
          }).then(function (response) {
            return _this.handleResponseAndCallback(response);
          }).catch(function (e) {
            _this.callback(e);
          });
        }
      }
    }

    /**
     * Send input
     *
     * @param payload
     */

  }, {
    key: 'input',
    value: function input(payload) {
      this.buffer.push(payload);
      if (this.buffer.length >= this.bufferSize) {
        this.flush();
      }
    }

    /**
     * Handle the response and calls the callback function
     *
     * @private
     * @param {Response} response
     */

  }, {
    key: 'handleResponseAndCallback',
    value: function handleResponseAndCallback(response) {
      return this.callback(response.ok === true ? null : response.status, response);
    }
  }]);
  return Emitter;
}();

var Version = '0.0.4';

var commonjsGlobal = typeof window !== 'undefined' ? window : typeof global !== 'undefined' ? global : typeof self !== 'undefined' ? self : {};

function unwrapExports (x) {
	return x && x.__esModule && Object.prototype.hasOwnProperty.call(x, 'default') ? x.default : x;
}

function createCommonjsModule(fn, module) {
	return module = { exports: {} }, fn(module, module.exports), module.exports;
}

var rng;

var crypto = commonjsGlobal.crypto || commonjsGlobal.msCrypto; // for IE 11
if (crypto && crypto.getRandomValues) {
  // WHATWG crypto-based RNG - http://wiki.whatwg.org/wiki/Crypto
  // Moderately fast, high quality
  var _rnds8 = new Uint8Array(16);
  rng = function whatwgRNG() {
    crypto.getRandomValues(_rnds8);
    return _rnds8;
  };
}

if (!rng) {
  // Math.random()-based (RNG)
  //
  // If all else fails, use Math.random().  It's fast, but is of unspecified
  // quality.
  var  _rnds = new Array(16);
  rng = function() {
    for (var i = 0, r; i < 16; i++) {
      if ((i & 0x03) === 0) r = Math.random() * 0x100000000;
      _rnds[i] = r >>> ((i & 0x03) << 3) & 0xff;
    }

    return _rnds;
  };
}

var rngBrowser = rng;

//     uuid.js
//
//     Copyright (c) 2010-2012 Robert Kieffer
//     MIT License - http://opensource.org/licenses/mit-license.php

// Unique ID creation requires a high quality random # generator.  We feature
// detect to determine the best RNG source, normalizing to a function that
// returns 128-bits of randomness, since that's what's usually required


// Maps for number <-> hex string conversion
var _byteToHex = [];
var _hexToByte = {};
for (var i = 0; i < 256; i++) {
  _byteToHex[i] = (i + 0x100).toString(16).substr(1);
  _hexToByte[_byteToHex[i]] = i;
}

// **`parse()` - Parse a UUID into it's component bytes**
function parse$1(s, buf, offset) {
  var i = (buf && offset) || 0, ii = 0;

  buf = buf || [];
  s.toLowerCase().replace(/[0-9a-f]{2}/g, function(oct) {
    if (ii < 16) { // Don't overflow!
      buf[i + ii++] = _hexToByte[oct];
    }
  });

  // Zero out remaining bytes if string was short
  while (ii < 16) {
    buf[i + ii++] = 0;
  }

  return buf;
}

// **`unparse()` - Convert UUID byte array (ala parse()) into a string**
function unparse(buf, offset) {
  var i = offset || 0, bth = _byteToHex;
  return  bth[buf[i++]] + bth[buf[i++]] +
          bth[buf[i++]] + bth[buf[i++]] + '-' +
          bth[buf[i++]] + bth[buf[i++]] + '-' +
          bth[buf[i++]] + bth[buf[i++]] + '-' +
          bth[buf[i++]] + bth[buf[i++]] + '-' +
          bth[buf[i++]] + bth[buf[i++]] +
          bth[buf[i++]] + bth[buf[i++]] +
          bth[buf[i++]] + bth[buf[i++]];
}

// **`v1()` - Generate time-based UUID**
//
// Inspired by https://github.com/LiosK/UUID.js
// and http://docs.python.org/library/uuid.html

// random #'s we need to init node and clockseq
var _seedBytes = rngBrowser();

// Per 4.5, create and 48-bit node id, (47 random bits + multicast bit = 1)
var _nodeId = [
  _seedBytes[0] | 0x01,
  _seedBytes[1], _seedBytes[2], _seedBytes[3], _seedBytes[4], _seedBytes[5]
];

// Per 4.2.2, randomize (14 bit) clockseq
var _clockseq = (_seedBytes[6] << 8 | _seedBytes[7]) & 0x3fff;

// Previous uuid creation time
var _lastMSecs = 0, _lastNSecs = 0;

// See https://github.com/broofa/node-uuid for API details
function v1(options, buf, offset) {
  var i = buf && offset || 0;
  var b = buf || [];

  options = options || {};

  var clockseq = options.clockseq !== undefined ? options.clockseq : _clockseq;

  // UUID timestamps are 100 nano-second units since the Gregorian epoch,
  // (1582-10-15 00:00).  JSNumbers aren't precise enough for this, so
  // time is handled internally as 'msecs' (integer milliseconds) and 'nsecs'
  // (100-nanoseconds offset from msecs) since unix epoch, 1970-01-01 00:00.
  var msecs = options.msecs !== undefined ? options.msecs : new Date().getTime();

  // Per 4.2.1.2, use count of uuid's generated during the current clock
  // cycle to simulate higher resolution clock
  var nsecs = options.nsecs !== undefined ? options.nsecs : _lastNSecs + 1;

  // Time since last uuid creation (in msecs)
  var dt = (msecs - _lastMSecs) + (nsecs - _lastNSecs)/10000;

  // Per 4.2.1.2, Bump clockseq on clock regression
  if (dt < 0 && options.clockseq === undefined) {
    clockseq = clockseq + 1 & 0x3fff;
  }

  // Reset nsecs if clock regresses (new clockseq) or we've moved onto a new
  // time interval
  if ((dt < 0 || msecs > _lastMSecs) && options.nsecs === undefined) {
    nsecs = 0;
  }

  // Per 4.2.1.2 Throw error if too many uuids are requested
  if (nsecs >= 10000) {
    throw new Error('uuid.v1(): Can\'t create more than 10M uuids/sec');
  }

  _lastMSecs = msecs;
  _lastNSecs = nsecs;
  _clockseq = clockseq;

  // Per 4.1.4 - Convert from unix epoch to Gregorian epoch
  msecs += 12219292800000;

  // `time_low`
  var tl = ((msecs & 0xfffffff) * 10000 + nsecs) % 0x100000000;
  b[i++] = tl >>> 24 & 0xff;
  b[i++] = tl >>> 16 & 0xff;
  b[i++] = tl >>> 8 & 0xff;
  b[i++] = tl & 0xff;

  // `time_mid`
  var tmh = (msecs / 0x100000000 * 10000) & 0xfffffff;
  b[i++] = tmh >>> 8 & 0xff;
  b[i++] = tmh & 0xff;

  // `time_high_and_version`
  b[i++] = tmh >>> 24 & 0xf | 0x10; // include version
  b[i++] = tmh >>> 16 & 0xff;

  // `clock_seq_hi_and_reserved` (Per 4.2.2 - include variant)
  b[i++] = clockseq >>> 8 | 0x80;

  // `clock_seq_low`
  b[i++] = clockseq & 0xff;

  // `node`
  var node = options.node || _nodeId;
  for (var n = 0; n < 6; n++) {
    b[i + n] = node[n];
  }

  return buf ? buf : unparse(b);
}

// **`v4()` - Generate random UUID**

// See https://github.com/broofa/node-uuid for API details
function v4(options, buf, offset) {
  // Deprecated - 'format' argument, as supported in v1.2
  var i = buf && offset || 0;

  if (typeof(options) == 'string') {
    buf = options == 'binary' ? new Array(16) : null;
    options = null;
  }
  options = options || {};

  var rnds = options.random || (options.rng || rngBrowser)();

  // Per 4.4, set bits for version and `clock_seq_hi_and_reserved`
  rnds[6] = (rnds[6] & 0x0f) | 0x40;
  rnds[8] = (rnds[8] & 0x3f) | 0x80;

  // Copy bytes to buffer, if provided
  if (buf) {
    for (var ii = 0; ii < 16; ii++) {
      buf[i + ii] = rnds[ii];
    }
  }

  return buf || unparse(rnds);
}

// Export public API
var uuid = v4;
uuid.v1 = v1;
uuid.v4 = v4;
uuid.parse = parse$1;
uuid.unparse = unparse;

var uuid_1 = uuid;

var base64 = createCommonjsModule(function (module, exports) {
Object.defineProperty(exports, "__esModule", { value: true });
function base64encode(data) {
    var b64 = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/=';
    var o1, o2, o3, h1, h2, h3, h4, bits, i = 0, ac = 0, enc, tmp_arr = [];
    if (!data) {
        return data;
    }
    data = unescape(encodeURIComponent(data));
    do {
        o1 = data.charCodeAt(i++);
        o2 = data.charCodeAt(i++);
        o3 = data.charCodeAt(i++);
        bits = o1 << 16 | o2 << 8 | o3;
        h1 = bits >> 18 & 0x3f;
        h2 = bits >> 12 & 0x3f;
        h3 = bits >> 6 & 0x3f;
        h4 = bits & 0x3f;
        tmp_arr[ac++] = b64.charAt(h1) + b64.charAt(h2) + b64.charAt(h3) + b64.charAt(h4);
    } while (i < data.length);
    enc = tmp_arr.join('');
    var r = data.length % 3;
    return (r ? enc.slice(0, r - 3) : enc) + '==='.slice(r || 3);
}
exports.base64encode = base64encode;

});

unwrapExports(base64);
var base64_1 = base64.base64encode;

var payload = createCommonjsModule(function (module, exports) {
Object.defineProperty(exports, "__esModule", { value: true });

function base64urlencode(data) {
    if (!data) {
        return data;
    }
    var enc = base64.base64encode(data);
    return enc.replace(/=/g, '').replace(/\+/g, '-').replace(/\//g, '_');
}
function isNonEmptyJson(property) {
    if (!isJson(property)) {
        return false;
    }
    for (var key in property) {
        if (property.hasOwnProperty(key)) {
            return true;
        }
    }
    return false;
}
exports.isNonEmptyJson = isNonEmptyJson;
function isJson(property) {
    return (typeof property !== 'undefined' && property !== null &&
        (property.constructor === {}.constructor || property.constructor === [].constructor));
}
exports.isJson = isJson;
function payloadBuilder(base64Encode) {
    var dict = {};
    var add = function (key, value) {
        if (value != null && value !== '') {
            dict[key] = value;
        }
    };
    var addDict = function (dict) {
        for (var key in dict) {
            if (dict.hasOwnProperty(key)) {
                add(key, dict[key]);
            }
        }
    };
    var addJson = function (keyIfEncoded, keyIfNotEncoded, json) {
        if (isNonEmptyJson(json)) {
            var str = JSON.stringify(json);
            if (base64Encode) {
                add(keyIfEncoded, base64urlencode(str));
            }
            else {
                add(keyIfNotEncoded, str);
            }
        }
    };
    return {
        add: add,
        addDict: addDict,
        addJson: addJson,
        build: function () {
            return dict;
        }
    };
}
exports.payloadBuilder = payloadBuilder;

});

unwrapExports(payload);
var payload_1 = payload.isNonEmptyJson;
var payload_2 = payload.isJson;
var payload_3 = payload.payloadBuilder;

var core = createCommonjsModule(function (module, exports) {
Object.defineProperty(exports, "__esModule", { value: true });


function getTimestamp(tstamp) {
    if (tstamp == null) {
        return { type: 'dtm', value: new Date().getTime() };
    }
    else if (typeof tstamp === 'number') {
        return { type: 'dtm', value: tstamp };
    }
    else if (tstamp.type === 'ttm') {
        return { type: 'ttm', value: tstamp.value };
    }
    else {
        return { type: 'dtm', value: (tstamp.value || new Date().getTime()) };
    }
}
function trackerCore(base64, callback) {
    if (typeof base64 === 'undefined') {
        base64 = true;
    }
    var payloadPairs = {};
    function addPayloadPair(key, value) {
        payloadPairs[key] = value;
    }
    function removeEmptyProperties(eventJson, exemptFields) {
        var ret = {};
        exemptFields = exemptFields || {};
        for (var k in eventJson) {
            if (exemptFields[k] || (eventJson[k] !== null && typeof eventJson[k] !== 'undefined')) {
                ret[k] = eventJson[k];
            }
        }
        return ret;
    }
    function completeContexts(contexts) {
        if (contexts && contexts.length) {
            return {
                schema: 'iglu:com.snowplowanalytics.snowplow/contexts/jsonschema/1-0-0',
                data: contexts
            };
        }
    }
    function track(sb, context, tstamp) {
        sb.addDict(payloadPairs);
        sb.add('eid', uuid_1.v4());
        var timestamp = getTimestamp(tstamp);
        sb.add(timestamp.type, timestamp.value.toString());
        var wrappedContexts = completeContexts(context);
        if (wrappedContexts !== undefined) {
            sb.addJson('cx', 'co', wrappedContexts);
        }
        if (typeof callback === 'function') {
            callback(sb);
        }
        return sb;
    }
    function trackSelfDescribingEvent(properties, context, tstamp) {
        var sb = payload.payloadBuilder(base64);
        var ueJson = {
            schema: 'iglu:com.snowplowanalytics.snowplow/unstruct_event/jsonschema/1-0-0',
            data: properties
        };
        sb.add('e', 'ue');
        sb.addJson('ue_px', 'ue_pr', ueJson);
        return track(sb, context, tstamp);
    }
    return {
        setBase64Encoding: function (encode) {
            base64 = encode;
        },
        addPayloadPair: addPayloadPair,
        addPayloadDict: function (dict) {
            for (var key in dict) {
                if (dict.hasOwnProperty(key)) {
                    payloadPairs[key] = dict[key];
                }
            }
        },
        resetPayloadPairs: function (dict) {
            payloadPairs = payload.isJson(dict) ? dict : {};
        },
        setTrackerVersion: function (version) {
            addPayloadPair('tv', version);
        },
        setTrackerNamespace: function (name) {
            addPayloadPair('tna', name);
        },
        setAppId: function (appId) {
            addPayloadPair('aid', appId);
        },
        setPlatform: function (value) {
            addPayloadPair('p', value);
        },
        setUserId: function (userId) {
            addPayloadPair('uid', userId);
        },
        setScreenResolution: function (width, height) {
            addPayloadPair('res', width + 'x' + height);
        },
        setViewport: function (width, height) {
            addPayloadPair('vp', width + 'x' + height);
        },
        setColorDepth: function (depth) {
            addPayloadPair('cd', depth);
        },
        setTimezone: function (timezone) {
            addPayloadPair('tz', timezone);
        },
        setLang: function (lang) {
            addPayloadPair('lang', lang);
        },
        setIpAddress: function (ip) {
            addPayloadPair('ip', ip);
        },
        trackUnstructEvent: trackSelfDescribingEvent,
        trackSelfDescribingEvent: trackSelfDescribingEvent,
        trackPageView: function (pageUrl, pageTitle, referrer, context, tstamp) {
            var sb = payload.payloadBuilder(base64);
            sb.add('e', 'pv');
            sb.add('url', pageUrl);
            sb.add('page', pageTitle);
            sb.add('refr', referrer);
            return track(sb, context, tstamp);
        },
        trackPagePing: function (pageUrl, pageTitle, referrer, minXOffset, maxXOffset, minYOffset, maxYOffset, context, tstamp) {
            var sb = payload.payloadBuilder(base64);
            sb.add('e', 'pp');
            sb.add('url', pageUrl);
            sb.add('page', pageTitle);
            sb.add('refr', referrer);
            sb.add('pp_mix', minXOffset.toString());
            sb.add('pp_max', maxXOffset.toString());
            sb.add('pp_miy', minYOffset.toString());
            sb.add('pp_may', maxYOffset.toString());
            return track(sb, context, tstamp);
        },
        trackStructEvent: function (category, action, label, property, value, context, tstamp) {
            var sb = payload.payloadBuilder(base64);
            sb.add('e', 'se');
            sb.add('se_ca', category);
            sb.add('se_ac', action);
            sb.add('se_la', label);
            sb.add('se_pr', property);
            sb.add('se_va', (value == null ? undefined : value.toString()));
            return track(sb, context, tstamp);
        },
        trackEcommerceTransaction: function (orderId, affiliation, totalValue, taxValue, shipping, city, state, country, currency, context, tstamp) {
            var sb = payload.payloadBuilder(base64);
            sb.add('e', 'tr');
            sb.add("tr_id", orderId);
            sb.add("tr_af", affiliation);
            sb.add("tr_tt", totalValue);
            sb.add("tr_tx", taxValue);
            sb.add("tr_sh", shipping);
            sb.add("tr_ci", city);
            sb.add("tr_st", state);
            sb.add("tr_co", country);
            sb.add("tr_cu", currency);
            return track(sb, context, tstamp);
        },
        trackEcommerceTransactionItem: function (orderId, sku, name, category, price, quantity, currency, context, tstamp) {
            var sb = payload.payloadBuilder(base64);
            sb.add("e", "ti");
            sb.add("ti_id", orderId);
            sb.add("ti_sk", sku);
            sb.add("ti_nm", name);
            sb.add("ti_ca", category);
            sb.add("ti_pr", price);
            sb.add("ti_qu", quantity);
            sb.add("ti_cu", currency);
            return track(sb, context, tstamp);
        },
        trackScreenView: function (name, id, context, tstamp) {
            return trackSelfDescribingEvent({
                schema: 'iglu:com.snowplowanalytics.snowplow/screen_view/jsonschema/1-0-0',
                data: removeEmptyProperties({
                    name: name,
                    id: id
                })
            }, context, tstamp);
        },
        trackLinkClick: function (targetUrl, elementId, elementClasses, elementTarget, elementContent, context, tstamp) {
            var eventJson = {
                schema: 'iglu:com.snowplowanalytics.snowplow/link_click/jsonschema/1-0-1',
                data: removeEmptyProperties({
                    targetUrl: targetUrl,
                    elementId: elementId,
                    elementClasses: elementClasses,
                    elementTarget: elementTarget,
                    elementContent: elementContent
                })
            };
            return trackSelfDescribingEvent(eventJson, context, tstamp);
        },
        trackAdImpression: function (impressionId, costModel, cost, targetUrl, bannerId, zoneId, advertiserId, campaignId, context, tstamp) {
            var eventJson = {
                schema: 'iglu:com.snowplowanalytics.snowplow/ad_impression/jsonschema/1-0-0',
                data: removeEmptyProperties({
                    impressionId: impressionId,
                    costModel: costModel,
                    cost: cost,
                    targetUrl: targetUrl,
                    bannerId: bannerId,
                    zoneId: zoneId,
                    advertiserId: advertiserId,
                    campaignId: campaignId
                })
            };
            return trackSelfDescribingEvent(eventJson, context, tstamp);
        },
        trackAdClick: function (targetUrl, clickId, costModel, cost, bannerId, zoneId, impressionId, advertiserId, campaignId, context, tstamp) {
            var eventJson = {
                schema: 'iglu:com.snowplowanalytics.snowplow/ad_click/jsonschema/1-0-0',
                data: removeEmptyProperties({
                    targetUrl: targetUrl,
                    clickId: clickId,
                    costModel: costModel,
                    cost: cost,
                    bannerId: bannerId,
                    zoneId: zoneId,
                    impressionId: impressionId,
                    advertiserId: advertiserId,
                    campaignId: campaignId
                })
            };
            return trackSelfDescribingEvent(eventJson, context, tstamp);
        },
        trackAdConversion: function (conversionId, costModel, cost, category, action, property, initialValue, advertiserId, campaignId, context, tstamp) {
            var eventJson = {
                schema: 'iglu:com.snowplowanalytics.snowplow/ad_conversion/jsonschema/1-0-0',
                data: removeEmptyProperties({
                    conversionId: conversionId,
                    costModel: costModel,
                    cost: cost,
                    category: category,
                    action: action,
                    property: property,
                    initialValue: initialValue,
                    advertiserId: advertiserId,
                    campaignId: campaignId
                })
            };
            return trackSelfDescribingEvent(eventJson, context, tstamp);
        },
        trackSocialInteraction: function (action, network, target, context, tstamp) {
            var eventJson = {
                schema: 'iglu:com.snowplowanalytics.snowplow/social_interaction/jsonschema/1-0-0',
                data: removeEmptyProperties({
                    action: action,
                    network: network,
                    target: target
                })
            };
            return trackSelfDescribingEvent(eventJson, context, tstamp);
        },
        trackAddToCart: function (sku, name, category, unitPrice, quantity, currency, context, tstamp) {
            return trackSelfDescribingEvent({
                schema: 'iglu:com.snowplowanalytics.snowplow/add_to_cart/jsonschema/1-0-0',
                data: removeEmptyProperties({
                    sku: sku,
                    name: name,
                    category: category,
                    unitPrice: unitPrice,
                    quantity: quantity,
                    currency: currency
                })
            }, context, tstamp);
        },
        trackRemoveFromCart: function (sku, name, category, unitPrice, quantity, currency, context, tstamp) {
            return trackSelfDescribingEvent({
                schema: 'iglu:com.snowplowanalytics.snowplow/remove_from_cart/jsonschema/1-0-0',
                data: removeEmptyProperties({
                    sku: sku,
                    name: name,
                    category: category,
                    unitPrice: unitPrice,
                    quantity: quantity,
                    currency: currency
                })
            }, context, tstamp);
        },
        trackFormChange: function (formId, elementId, nodeName, type, elementClasses, value, context, tstamp) {
            return trackSelfDescribingEvent({
                schema: 'iglu:com.snowplowanalytics.snowplow/change_form/jsonschema/1-0-0',
                data: removeEmptyProperties({
                    formId: formId,
                    elementId: elementId,
                    nodeName: nodeName,
                    type: type,
                    elementClasses: elementClasses,
                    value: value
                }, { value: true })
            }, context, tstamp);
        },
        trackFormSubmission: function (formId, formClasses, elements, context, tstamp) {
            return trackSelfDescribingEvent({
                schema: 'iglu:com.snowplowanalytics.snowplow/submit_form/jsonschema/1-0-0',
                data: removeEmptyProperties({
                    formId: formId,
                    formClasses: formClasses,
                    elements: elements
                })
            }, context, tstamp);
        },
        trackSiteSearch: function (terms, filters, totalResults, pageResults, context, tstamp) {
            return trackSelfDescribingEvent({
                schema: 'iglu:com.snowplowanalytics.snowplow/site_search/jsonschema/1-0-0',
                data: removeEmptyProperties({
                    terms: terms,
                    filters: filters,
                    totalResults: totalResults,
                    pageResults: pageResults
                })
            }, context, tstamp);
        },
        trackConsentWithdrawn: function (all, id, version, name, description, context, tstamp) {
            var documentJson = {
                schema: 'iglu:com.snowplowanalytics.snowplow/consent_document/jsonschema/1-0-0',
                data: removeEmptyProperties({
                    id: id,
                    version: version,
                    name: name,
                    description: description
                })
            };
            return trackSelfDescribingEvent({
                schema: 'iglu:com.snowplowanalytics.snowplow/consent_withdrawn/jsonschema/1-0-0',
                data: removeEmptyProperties({
                    all: all
                })
            }, documentJson.data && context ? context.concat([documentJson]) : context, tstamp);
        },
        trackConsentGranted: function (id, version, name, description, expiry, context, tstamp) {
            var documentJson = {
                schema: 'iglu:com.snowplowanalytics.snowplow/consent_document/jsonschema/1-0-0',
                data: removeEmptyProperties({
                    id: id,
                    version: version,
                    name: name,
                    description: description,
                })
            };
            return trackSelfDescribingEvent({
                schema: 'iglu:com.snowplowanalytics.snowplow/consent_granted/jsonschema/1-0-0',
                data: removeEmptyProperties({
                    expiry: expiry,
                })
            }, context ? context.concat([documentJson]) : [documentJson], tstamp);
        }
    };
}
exports.trackerCore = trackerCore;

});

unwrapExports(core);
var core_1 = core.trackerCore;

var snowplowTrackerCore = createCommonjsModule(function (module, exports) {
Object.defineProperty(exports, "__esModule", { value: true });

exports.trackerCore = core.trackerCore;

});

unwrapExports(snowplowTrackerCore);
var snowplowTrackerCore_1 = snowplowTrackerCore.trackerCore;

var DEFAULT_PLATFORM = 'mob';
var DEFAULT_TRACKER_VERSION = 'react-' + Version;

var Tracker = function () {
  /**
   * Snowplow React Native Tracker
   *
   * @param {Emitter|Array<Emitter>} emitters The emitter or emitters to which events will be sent
   * @param {String} namespace The namespace of the tracker
   * @param {String} appId The application ID
   * @param {Boolean} encodeBase64 Whether unstructured events and custom contexts should be base 64 encoded
   */
  function Tracker(emitters, namespace, appId) {
    var _this = this;

    var encodeBase64 = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : true;
    classCallCheck(this, Tracker);

    emitters = emitters instanceof Array ? emitters : [emitters];
    encodeBase64 = encodeBase64 !== false;

    // configure core tracker
    this.core = snowplowTrackerCore_1(encodeBase64, function (payload) {
      var builtPayload = payload.build();
      for (var i = 0; i < _this.emitters.length; i++) {
        _this.emitters[i].input(builtPayload);
      }
    });

    this.emitters = emitters;

    // set defaults
    this.core.setPlatform(DEFAULT_PLATFORM);
    this.core.setTrackerVersion(DEFAULT_TRACKER_VERSION);
    this.core.setTrackerNamespace(namespace);
    this.core.setAppId(appId);
  }

  /**
   * Set the domain_user_id
   *
   * @param {String} duid
   */


  createClass(Tracker, [{
    key: 'setDomainUserId',
    value: function setDomainUserId(duid) {
      this.core.addPayloadPair('duid', duid);
    }

    // expose other core tracker methods
    /* eslint-disable */

  }, {
    key: 'setPlatform',
    value: function setPlatform(val) {
      this.core.setPlatform(val);
    }
  }, {
    key: 'setUserId',
    value: function setUserId(val) {
      this.core.setUserId(val);
    }
  }, {
    key: 'setScreenResolution',
    value: function setScreenResolution(width, height) {
      this.core.setScreenResolution(width, height);
    }
  }, {
    key: 'setViewport',
    value: function setViewport(width, height) {
      this.core.setViewport(width, height);
    }
  }, {
    key: 'setColorDepth',
    value: function setColorDepth(val) {
      this.core.setColorDepth(val);
    }
  }, {
    key: 'setTimezone',
    value: function setTimezone(val) {
      this.core.setTimezone(val);
    }
  }, {
    key: 'setLang',
    value: function setLang(val) {
      this.core.setLang(val);
    }
  }, {
    key: 'trackScreenView',
    value: function trackScreenView(name, id, context, tstamp) {
      this.core.trackScreenView(name, id, context, tstamp);
    }
  }, {
    key: 'trackPageView',
    value: function trackPageView(pageUrl, pageTitle, referrer, context, tstamp) {
      this.core.trackPageView(pageUrl, pageTitle, referrer, context, tstamp);
    }
  }, {
    key: 'trackPagePing',
    value: function trackPagePing(pageUrl, pageTitle, referrer, minXOffset, maxXOffset, minYOffset, maxYOffset, context, tstamp) {
      this.core.trackPagePing(pageUrl, pageTitle, referrer, minXOffset, maxXOffset, minYOffset, maxYOffset, context, tstamp);
    }
  }, {
    key: 'trackStructEvent',
    value: function trackStructEvent(category, action, label, property, value, context, tstamp) {
      this.core.trackStructEvent(category, action, label, property, value, context, tstamp);
    }
  }, {
    key: 'trackUnstructEvent',
    value: function trackUnstructEvent(properties, context, tstamp) {
      this.core.trackUnstructEvent(properties, context, tstamp);
    }
    /* eslint-enable */

  }]);
  return Tracker;
}();

exports.Emitter = Emitter;
exports.Tracker = Tracker;
exports.Version = Version;
//# sourceMappingURL=index.js.map
